
import XMonad
--import XMonad.Config.Gnome

import Data.Monoid
import System.Exit

-- to get mouse move with window
import XMonad.Actions.UpdatePointer

-- needed for the panel toggle keybind
import XMonad.Hooks.ManageDocks

-- not sure if this is redundant with Xfce import
import XMonad.Config.Desktop

-- based on http://www.haskell.org/haskellwiki/Xmonad/Using_xmonad_in_XFCE
import XMonad.Hooks.EwmhDesktops
import XMonad.Config.Xfce
import XMonad.Hooks.ManageDocks
import XMonad.Layout.NoFrillsDecoration

import qualified XMonad.StackSet as W
import qualified Data.Map        as M

import XMonad.Actions.CycleWindows    --  Reset the layouts on the current workspace to default

import XMonad.Hooks.SetWMName


-- /usr/share/xmonad/xmonad.hs on fedora has integration with gnome etc. but I
-- had lots of problems with gnome integration.
myL = noFrillsDeco shrinkText def (layoutHook def)

main = xmonad $ defaults
--main = xmonad def { layoutHook = myL } $ defaults


defaults = desktopConfig
       {
         terminal            = "konsole"
       , borderWidth        = 0
       , modMask            = mod4Mask
       , keys               = myKeys
       --, logHook            = myLogHook <+> logHook desktopConfig
       , mouseBindings      = myMouseBindings


    --   manageDocks & ewmh stuff is based on xfce integration wiki page
       , manageHook    = manageDocks <+> myManageHook <+> manageHook defaultConfig
       , logHook    = ewmhDesktopsLogHook
-- todo, look into the suggested avoidStruts. my nieve attempt to integrate it failed
       , layoutHook         = myLayout
       , handleEventHook = ewmhDesktopsEventHook
-- this makes some java programs work, like intellij idea,
-- but it also makes the ui buggy for some gtk apps, like transmission
-- and gnome-terminal
-- https://bbs.archlinux.org/viewtopic.php?pid=744577
       , startupHook = setWMName "LG3D"
       -- combining the 2 with <+> didn't make java apps work.
       --, startupHook = ewmhDesktopsStartup

}


-- for android emulator buggy window
myManageHook = composeAll [ className =? "emulator64-arm" --> unfloat]
    where unfloat = ask >>= doF . W.sink

-- this is a nice idea but needs some tweaking cuz i don't want it to
--  have an affect when switching to a floating window,
--  and I remember sometimes being stuck in floating dialog boxes
-- myLogHook = updatePointer (Relative 0.5 0.5)



myKeys conf@(XConfig {XMonad.modMask = modm}) = M.fromList $


-- Move focus to the previous window
--    , ((modm,               xK_q     ), windows W.focusUp  )


    [ ((modm, xK_q), io (exitWith ExitSuccess)) -- %! Quit xmonad

    , ((modm, xK_2), spawn "pavucontrol")
--    , ((modm, xK_3), spawn "firefox  -no-remote -P sfw")
--    todo: clicking links in other applications won't open in
--    a firefox opened with -no-remote, so we have to hardcode a
--    primary firefox profile here. I'd rather have a command which
--    sets which is the pimary for opening links. It could alter a file,
--    and we could launch firefox with a wrapper script which looks at that file.
    , ((modm, xK_3), spawn "firefox -P sfw")
    , ((modm, xK_4), spawn "firefox -no-remote -P firefox-main-profile")

    , ((modm, xK_5), spawn "/a/exe/input-setup l")
    , ((modm, xK_1), spawn "/a/exe/input-setup k")

    -- we would really want to keep track of which workspace is darkened,
    -- and change things depending on where that workspace is

      -- fedora vs debian ordering
--    , ((modm, xK_w),  spawn "redshift.sh 0")
--    , ((modm, xK_e),  spawn "redshift.sh 2")
--    , ((modm, xK_r),  spawn "redshift.sh 1")
    , ((modm, xK_w),  spawn "/a/bin/redshift.sh 2")
    , ((modm, xK_e),  spawn "/a/bin/redshift.sh 1")
    , ((modm, xK_r),  spawn "/a/bin/redshift.sh 0")

  -- Rotate through the available layout algorithms
    , ((modm,               xK_t ), sendMessage NextLayout)

    , ((modm              , xK_Home     ), sendMessage ToggleStruts)

-- disabled default of cycling windows. instead use traditional stack based alt-tab behavior
-- todo, this locks up everytyhing, and the only thing that works is
-- futher cycle key presses until i restart X
  --  , ((modm,               xK_g   ), cycleRecentWindows [xK_Super_L,xK_Super_R] xK_g xK_Tab)

    -- close focused window
    , ((modm , xK_c     ), kill)


    --  Reset the layouts on the current workspace to default
    , ((modm, xK_b ), setLayout $ XMonad.layoutHook conf)


    -- todo, make the pointer follow the window for these

    -- fedora's order
--    , ((modm              , xK_Delete), screenWorkspace 0 >>= flip whenJust (windows . W.shift))
--    , ((modm              , xK_parenleft), screenWorkspace 2 >>= flip whenJust (windows . W.shift))
--    , ((modm              , xK_parenright), screenWorkspace 1 >>= flip whenJust (windows . W.shift))
-- debian's order
    , ((modm              , xK_Delete), screenWorkspace 2 >>= flip whenJust (windows . W.shift))
    , ((modm              , xK_parenleft), screenWorkspace 1 >>= flip whenJust (windows . W.shift))
    , ((modm              , xK_parenright), screenWorkspace 0 >>= flip whenJust (windows . W.shift))

    -- Move focus to the next window
    , ((0,               xK_F8   ), windows W.focusDown)

    , ((modm              , xK_6), windows $ W.greedyView "6")

-- Swap the focused window with the next window
    , ((modm, xK_asterisk     ), windows W.swapDown  )

    -- Swap the focused window with the previous window
    , ((modm, xK_9     ), windows W.swapUp    )


-- fedora's order
--    , ((modm              , xK_u), screenWorkspace 0 >>= flip whenJust (windows . W.view))
--    , ((modm              , xK_i), screenWorkspace 2 >>= flip whenJust (windows . W.view))
--    , ((modm              , xK_o), screenWorkspace 1 >>= flip whenJust (windows . W.view))
-- debian's order
    , ((modm              , xK_u), screenWorkspace 2 >>= flip whenJust (windows . W.view))
    , ((modm              , xK_i), screenWorkspace 1 >>= flip whenJust (windows . W.view))
    , ((modm              , xK_o), screenWorkspace 0 >>= flip whenJust (windows . W.view))


     -- Restart xmonad
    , ((modm              , xK_p), spawn "xmonad --recompile; xmonad --restart")
   -- Push window back into tiling
    , ((modm,               xK_h    ), withFocused $ windows . W.sink)

    -- this working is dependent upon env vars set in distro-start
    , ((modm,               xK_j     ), spawn "emacsclient -c")


    -- launch a terminal
    , ((modm, xK_k), spawn $ XMonad.terminal conf)


    , ((modm, xK_l), spawn "dmenu_run")

    -- Expand the master area
    , ((modm,               xK_n     ), sendMessage Expand)

   -- Shrink the master area
    , ((modm,               xK_m    ), sendMessage Shrink)

    -- Move focus to the master window
    , ((modm,               xK_y     ), windows W.focusMaster  )



    -- Increment the number of windows in the master area
    , ((modm              , xK_comma ), sendMessage (IncMasterN 1))

    -- Deincrement the number of windows in the master area
    , ((modm              , xK_period), sendMessage (IncMasterN (-1)))

    -- Swap the focused window and the master window
    --, ((modm,               xK_Return), windows W.swapMaster)
-- in recent versions, a different name works better
     , ((modm,               xK_Return), windows W.shiftMaster)

    ]
    ++



    -- open workspace, and send to workspace with same key plus shift
    [((m .|. modm, k), windows $ f i)
         | (i, k) <- zip (XMonad.workspaces conf) [xK_f, xK_d, xK_s, xK_a, xK_z, xK_grave, xK_7, xK_0]
    , (f, m) <- [(W.greedyView, 0), (W.shift, shiftMask)]]
--        fedora's default ordering of workspaces
 --         xK_s, xK_f, xK_d,




myMouseBindings (XConfig {XMonad.modMask = modm}) = M.fromList $

    -- mod-button1, Set the window to floating mode and move by dragging
    [ ((modm, button1), (\w -> focus w >> mouseMoveWindow w
                                       >> windows W.shiftMaster))

    -- mod-button2, Raise the window to the top of the stack
    , ((modm, button2), (\w -> focus w >> windows W.shiftMaster))

    -- mod-button3, Set the window to floating mode and resize by dragging
    , ((modm, button3), (\w -> focus w >> mouseResizeWindow w
                                       >> windows W.shiftMaster))

    -- you may also bind events to the mouse scroll wheel (button4 and button5)
    ]

myLayout = desktopLayoutModifiers $ Full ||| Mirror tiled ||| tiled
  where
    -- default tiling algorithm partitions the screen into two panes
    tiled   = Tall nmaster delta ratio

    -- The default number of windows in the master pane
    nmaster = 1

    -- Default proportion of screen occupied by master pane
    ratio   = 1/2

    -- Percent of screen to increment by when resizing panes
    delta   = 3/100
